#!/usr/bin/env python

#
# Author: Pawel Stolowski <pawel.stolowski@ubuntu.com>
#
# Released under the terms of GNU General Public License v3
#

from gi.repository import Unity
from gi.repository import GLib, Gio, GObject
from subprocess import Popen, PIPE
import os
import re

DBUS_NAME = 'net.launchpad.lens.manpages'
DBUS_PATH = '/net/launchpad/lens/manpages'
SCOPE_DBUS_PATH = '/net/launchpad/lens/scope/manpages'
UNKNOWN_MAN_CATEGORY = 9

class ManPagesScope(Unity.Scope):
	def __init__(self):
		super(ManPagesScope, self).__init__(dbus_path=SCOPE_DBUS_PATH)
		self.connect('search-changed', self._on_search_changed)
		self.connect('filters-changed', self._on_search_changed)
		self.connect('activate-uri', self.activate_uri)
		self.icon = Gio.ThemedIcon.new('help-contents').to_string()
		self.props.search_in_global = False
        #
        # regex_apropos for capturing 'apropos' results: manpage name (section)  - short description
		self.regex_apropos = re.compile('^(.+?)\s+\((\d+)\)\s+-\s(.+?)$')
		self.regex_section2cat = re.compile('^.*?(\d+)')

	def do_apropos(self, search_string):
		"""
		   	Run 'apropos <search string>' and parse, then return its results
		"""
		result = []
		exact_matches_count = 0
		partial_matches_count = 500
		other_matches_count = 1000
		score = 0
		if len(search_string) > 2:   # don't run apropos for strings shorter than 3 chars
			apropos = Popen(["apropos", search_string], stdout=PIPE)
			os.waitpid(apropos.pid, 0)
			out = apropos.communicate()[0]
			for line in out.split("\n"):
				m = self.regex_apropos.match(line)
				if m:
					#
					# append (score, man page name, section, description) tuple
					if m.group(1) == search_string:
						score = exact_matches_count
						exact_matches_count += 1
					elif m.group(1).startswith(search_string):
						score = partial_matches_count
						partial_matches_count += 1
					else:
					 	score = other_matches_count
						other_matches_count += 1
					result.append((score, m.group(1), m.group(2), m.group(3)))
		return result

	def man_cmp(self, searchstr, m1, m2):
		return cmp(m1[0], m2[0])

	def activate_uri(self, scope, uri):
		return Unity.ActivationResponse.new(Unity.HandledType.HIDE_DASH, "'uri'")

	def _on_search_changed(self, scope, search, search_type, cancellable):
		search_str = search.props.search_string
		res = sorted(self.do_apropos(search_str), cmp = lambda m1, m2: self.man_cmp(search_str, m1, m2))
		self._update_model(search.props.results_model, res)
		search.emit('finished')

	def build_path(self, name, section):
		return name + '(' + section + ')'

	def man_section_to_category(self, section):
		m = self.regex_section2cat.match(section)
		if m:
			try:
				return int(m.group(1)) - 1
			except:
				pass
		return UNKNOWN_MAN_CATEGORY

	def _update_model(self, model, result):
		model.clear()
		for res in result:
			uri = 'man:' + self.build_path(res[1], res[2])
			model.append(
                    uri,
					self.icon,
					self.man_section_to_category(res[2]),
					'x-scheme-handler/man',
					res[1] + ' (' + res[2] + ')',
					res[3],
				    uri	
					)

if __name__ == '__main__':
	session_bus_connection = Gio.bus_get_sync(Gio.BusType.SESSION, None)
	session_bus = Gio.DBusProxy.new_sync(session_bus_connection, 0, None, 'org.freedesktop.DBus', '/org/freedesktop/dbus', 'org.freedesktop.DBus', None,)
	res =session_bus.RequestName('(su)', DBUS_NAME, 0x4)

	lens = Unity.Lens.new(DBUS_PATH, 'manpages')
	lens.props.search_hint = "Search man pages"
	lens.props.visible = True
	lens.props.search_in_global = False
	icon = Gio.ThemedIcon.new("help-contents")
	lens.props.categories = [
				Unity.Category.new("General commands", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("System calls", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Library functions", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Special files", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("File formats and conventions", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Games and screensavers", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Miscellanea", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("System administration commands and daemons", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Kernel routines", icon, Unity.CategoryRenderer.HORIZONTAL_TILE),
				Unity.Category.new("Other manual pages", icon, Unity.CategoryRenderer.HORIZONTAL_TILE)
                ]
	lens.props.filters = []

	lens.add_local_scope(ManPagesScope())
	lens.export()

	GObject.MainLoop().run()
